//                                               -*- C++ -*-
/**
 *  @file  t_IsoProbabilisticTransformation_EllipticalCopula.cxx
 *  @brief The test file of class IsoProbabilisticTransformation for standard methods
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2008-06-26 13:50:17 +0200 (jeu 26 jun 2008) $
 *  Id:      $Id: t_IsoProbabilisticTransformation_EllipticalCopula.cxx 862 2008-06-26 11:50:17Z dutka $
 */
#include <iostream>
#include <sstream>
#include <cmath>
#include <iomanip>
#include "OT.hxx"
#include "OTtestcode.hxx"
#include "Normal.hxx"
#include "Uniform.hxx"
#include "Gamma.hxx"
#include "TruncatedNormal.hxx"
#include "NormalCopula.hxx"
#include "NumericalPoint.hxx"
#include "NumericalSample.hxx"
#include "CorrelationMatrix.hxx"
#include "IdentityMatrix.hxx"
#include "Distribution.hxx"
#include "CenteredFiniteDifferenceGradient.hxx"
#include "CenteredFiniteDifferenceHessian.hxx"
#include "ComposedDistribution.hxx"
#include "IndependentCopula.hxx"

using namespace OT;
using namespace OT::Test;
using namespace OT::Base::Common;
using namespace OT::Base::Type;
using namespace OT::Base::Stat;
using namespace OT::Base::Diff;
using namespace OT::Uncertainty::Model;
using namespace OT::Uncertainty::Distribution;

typedef Distribution::IsoProbabilisticTransformation IsoProbabilisticTransformation;
typedef Distribution::InverseIsoProbabilisticTransformation InverseIsoProbabilisticTransformation;

Matrix clean(Matrix in)
{
  UnsignedLong rowDim(in.getNbRows());
  UnsignedLong colDim(in.getNbColumns());
  for(UnsignedLong i = 0; i < rowDim; i++)
    for(UnsignedLong j = 0; j < colDim; j++)
      {
	in(i, j) = 1.e-6*round(1.e6*in(i, j));
	if (fabs(in(i, j)) < 1.e-6) in(i, j) = 0.0;
      }
  return in;
}

SymmetricTensor clean(SymmetricTensor in)
{
  UnsignedLong rowDim(in.getNbRows());
  UnsignedLong colDim(in.getNbColumns());
  UnsignedLong sheetDim(in.getNbSheets());
  for(UnsignedLong i = 0; i < rowDim; i++)
    for(UnsignedLong j = 0; j < colDim; j++)
      for (UnsignedLong k = 0; k < sheetDim; k++)
	{
	  in(i, j, k) = 1.e-4 * round(1.e4*in(i, j, k));
	  if (fabs(in(i, j, k)) < 1.e-4) in(i, j, k) = 0.0;
	}
  return in;
}

void print(NumericalPoint point)
{
  UnsignedLong dimension(point.getDimension());
  std::cout << "[";
  for (UnsignedLong i = 0; i < dimension; i++)
    {
      std::cout << ((i == 0) ? "":", ");
      std::cout << std::setprecision(16) << point[i];
    }
  std::cout << "]" << std::endl;
}

void print(Matrix matrix)
{
  UnsignedLong rowDimension(matrix.getNbRows());
  UnsignedLong colDimension(matrix.getNbColumns());
  for (UnsignedLong i = 0; i < rowDimension; i++)
    {
      std::cout << "|";
      for (UnsignedLong j = 0; j < colDimension; j++)
	{
          std::cout << ((j == 0) ? "":", ");
	  std::cout << std::setw(21) << std::setprecision(16) << matrix(i, j);
	}
      std::cout << "|" << std::endl;
    }
}

void print(SymmetricTensor tensor)
{
  UnsignedLong sheetDimension(tensor.getNbSheets());
  for (UnsignedLong k = 0; k < sheetDimension; k++)
    {
      print(Matrix(tensor.getSheet(k)));
      std::cout << std::endl;
    }
  std::cout << "]" << std::endl;
}

int main(int argc, char *argv[])
{
  TESTPREAMBLE;
  setRandomGenerator();

  try {


    NumericalPoint meanPoint(1);
    NumericalPoint sigma(1);
    CorrelationMatrix R(1);
    R(0, 0) = 1.0;
    // Create a collection of distribution
    ComposedDistribution::DistributionCollection aCollection;

    aCollection.add(Uniform(-1.0, 2.0));
    aCollection.add(Gamma(2.0, 2.0, 0.0));
    aCollection.add(TruncatedNormal(2.0, 5.0, -1.0, 2.0));

    UnsignedLong dim(aCollection.getSize());

    // Create a copula
    CorrelationMatrix RCopula(dim);
    for (UnsignedLong i = 0; i < dim; i++)
      {
	for (UnsignedLong j = 0; j < i; j++)
	  {
	    RCopula(i, j) = (i + j + 1.0) / (2.0 * dim);
	  }
      }
    // Instanciate one distribution object
    ComposedDistribution::DistributionCollection testDistribution(2);
    testDistribution[0] = ComposedDistribution(aCollection, NormalCopula(CorrelationMatrix(dim)));
    testDistribution[1] = ComposedDistribution(aCollection, NormalCopula(RCopula));
    for (UnsignedLong i = 0; i < 2; i++)
      {
	Distribution distribution = testDistribution[i];
	// Test for sampling
	UnsignedLong size = 10000;
	NumericalSample sample = distribution.getNumericalSample( size );
	std::cout << "sample first=" << sample[0] << " last=" << sample[size - 1] << std::endl;
	// Should be close to [0.5, 1, 0.544439]
	std::cout << "sample mean=" << sample.computeMean() << std::endl;
	// Should be close to [0.75, 0, 0, 0, 0.5, 0, 0, 0, 0.739866] for the independent copula
	std::cout << "sample covariance=" << sample.computeCovariance() << std::endl;

	IsoProbabilisticTransformation transform(distribution.getIsoProbabilisticTransformation());
	std::cout << "isoprobabilistic transformation=" << transform << std::endl;
	NumericalSample transformedSample(transform(sample));
	std::cout << "transformed sample first=" << transformedSample[0] << " last=" << transformedSample[size - 1] << std::endl;
	std::cout << "transformed sample mean=" << transformedSample.computeMean() << std::endl;
	std::cout << "transformed sample covariance=" << transformedSample.computeCovariance() << std::endl;

	// Test for evaluation
	InverseIsoProbabilisticTransformation inverseTransform(distribution.getInverseIsoProbabilisticTransformation());
	std::cout << "inverse isoprobabilistic transformation=" << inverseTransform << std::endl;
	NumericalSample transformedBackSample(inverseTransform(transformedSample));
	std::cout << "transformed back sample first=" << transformedBackSample[0] << " last=" << transformedBackSample[size - 1] << std::endl;
	std::cout << "transformed back sample mean=" << transformedBackSample.computeMean() << std::endl;
	std::cout << "transformed back sample covariance=" << transformedBackSample.computeCovariance() << std::endl;
	NumericalPoint point(dim, 1.0);
	std::cout << "point=" << point << std::endl;
	NumericalPoint transformedPoint(transform(point));
	std::cout << "transform value at point        =" << transformedPoint << std::endl;
	std::cout << "transform gradient at point     =" << clean(transform.gradient(point)) << std::endl;
	std::cout << "transform gradient at point (FD)=" << clean(CenteredFiniteDifferenceGradient(1.0e-5, transform.getEvaluationImplementation()).gradient(point)) << std::endl;
	std::cout << "transform hessian at point      =" << std::setprecision(4) << clean(transform.hessian(point)) << std::endl;
	std::cout << "transform hessian at point (FD) =" << std::setprecision(4) << clean(CenteredFiniteDifferenceHessian(1.0e-4, transform.getEvaluationImplementation()).hessian(point)) << std::endl;
	std::cout << "inverse transform value at transformed point        =" << inverseTransform(transformedPoint) << std::endl;
	std::cout << "inverse transform gradient at transformed point     =" << clean(inverseTransform.gradient(transformedPoint)) << std::endl;
	std::cout << "inverse transform gradient at transformed point (FD)=" << clean(CenteredFiniteDifferenceGradient(1.0e-5, inverseTransform.getEvaluationImplementation()).gradient(transformedPoint)) << std::endl;
	std::cout << "inverse transform hessian at transformed point      =" << std::setprecision(4) << clean(inverseTransform.hessian(transformedPoint)) << std::endl;
	std::cout << "inverse transform hessian at transformed point (FD) =" << std::setprecision(4) << clean(CenteredFiniteDifferenceHessian(1.0e-4, inverseTransform.getEvaluationImplementation()).hessian(transformedPoint)) << std::endl;

	// Test for parameters
	std::cout << "parameters gradient at point=" << transform.parametersGradient(point) << std::endl;
      }
  }
  catch (TestFailed & ex) {
    std::cerr << ex << std::endl;
    return ExitCode::Error;
  }


  return ExitCode::Success;
}
